c ----------------------------------------------------------
c  mkstr3 : Make color singlets for GGjets1 and 2
c         coded on 95/09/30 by K.K.
c         updated on 95/10/05
c         updated on 95/10/09 gamma->g + remnant(q+qbar)
c         updated on 96/01/19 to match ggps, e-->e part
c ----------------------------------------------------------
      subroutine mkstr3(mode,iprt,nstr,kstr,lstr,mstr,nastr,nret)

c Read /qcevnt/ and determine the connection of color flow.
c
c arguments:
c
c   mode : IN  : Mode selection
c                0 = Choose maximum pole term in hard scatt.
c                1 = Choose at random with weight // NOT NOW //
c   iprt : IN  : 0 = Error only, 1 = Report result, 2= Much report
c
c   nstr : OUT : Number of color singlets
c   kstr(i) : OUT : Type of i-th color singlet
c                   0 = gluonic loop
c                   1 = quark string
c   lstr(i) : OUT : Length of i-th color singlet (>=2)
c   mstr(i) : OUT : i-th color singlet starts from mstr(i)
c                   in nastr( ).
c   nastr(j): OUT : Addresses of particles in sequence.
c                   i-th color singlet is defined by
c                   j = mstr(i)..mstr(i)+lstr(i)-1
c   nret : OUT : 0 = Normal , 1 = Failed
c
c note : Arrays in arguments are properly declared in the
c        calling routine.
 
      IMPLICIT REAL*8(A-H,O-Z)
c
c   For debug. Normal user should keep KDBG=0.
      parameter(KDBG=0)

c   Particle code
      parameter(IPH=22, IQMX=6, IGLU=0, IELE=11)

c   Size of working array
      parameter(NNP=500,NNV=400)

c   Maximum number of retry when a single-gluon loop is found.
      parameter(NRETRY=4)

c   Arrays in arguments
      dimension kstr(1),lstr(1),mstr(1),nastr(1)

c   Working areas

c     ma(ii) = address of a color parton in /qcevnt/
c     mp(ii) = code of parton ; -/0/+ = qbar/g/q
c     ms(ii) = initial/final/link ; -1/0/1   //NOW NO -1//
      dimension ma(NNP),mp(NNP),ms(NNP)

c     mvx(k) = type of vertex ; 0/1/2/3 
c                = non-color/qqg/ggg/ggphoton
c     nvl(3,kk) = particles connected to vertex, 
c                 given by 'ii-address'.
c                 mvx=0 .. qbar, q, photon
c                     1 .. qbar, q, g
c                     2 .. g, g, g
c     mvxx(k) = working area for singlet construction
      dimension mvx(NNV),nvl(3,NNV),mvxx(NNV)

c     array to record vertices spawned from g-g-photon
      dimension nggph(8)
      
c Common area to record parton shower
      COMMON/qcevnt/partn(10,1000),listp(10,1000),NTOP
      common/modegg/modeg,neven

c ---------------------------------------
c  counter for retry
      mretry = 0
c    Jump point to repeat the trial
   99 continue
c ---------------------------------------
c  number of colored particles
      np = 0
c  number of final gluons
      ngl= 0
c  number of final quarks
      nqu= 0
c  number of final anti-quarks
      naq= 0
c  addresses of hard-scattering partons
      nhrd1 = 0
      nhrd2 = 0
c  counter to use nggph( )
      mggph = 0

c --- particle search and recording 
      do 100 n=1,ntop
        n1 = listp(1,n)
        n3 = listp(3,n)
        if ( n1.ge.-IQMX .and. n1.le.IQMX ) then
             np = np+1
             ma(np) = n
             mp(np) = n1
             if ( n3.eq.0 ) then
               ms(np) = 0
               if( n1.eq.IGLU ) ngl=ngl+1
               if( n1.gt.IGLU ) nqu=nqu+1
               if( n1.lt.IGLU ) naq=naq+1
             else
               ms(np) = 1
             end if
        end if

c     hard-scattering parton
        if( n3.eq.1 ) then
          if( nhrd1.eq.0 ) then
              nhrd1 = n
          else if( nhrd2.eq.0 ) then
              nhrd2 = n
          else
               if (modeg.eq.1.and. n.lt.ntop-1) then
               write(6,9002) n
               go to 9990
               end if
               if (modeg.eq.2.and. n.lt.ntop-3) then
               write(6,9002) n
               go to 9990
               end if
          end if
        end if
  100 continue

      if( nqu .ne. naq ) then
        write(6,9001) nqu,naq
        go to 9990
      end if

      if( iprt.gt.1 ) write(6,601) np,nqu,naq,ngl

      if( nhrd1.eq.0 .or. nhrd2.eq.0 ) then
        write(6,9003) 
        go to 9990
      end if

c ---------------------------------------
c  number of vertices
      nv = 0

c --- vertex search and recording (2-body only)
c     For q-qbar-photon (mvx=0) and q-qbar-g vertex (mvx=1),
c     nvl(1, ) = incoming q, nvl(2, ) = outgoing q.
c     For an internal quark line, incoming/outgoing follows 
c     the flow of fermion number.
c     At external end,
c      initial q , final qbar = incoming
c      final q , initial qbar = outgoing.

      do 110 n=1,ntop
        n1 = listp(1,n)
        n3 = listp(3,n)
        n4 = listp(4,n)
        if ( n3.eq.2 ) then
c            2-body branch, child-particles
           nc1 = n+n4
           nc2 = n+n4+1
           ncp1= listp(1,nc1)
           ncp2= listp(1,nc2)

c        photon -->
           if( n1.eq.IPH ) then
             if( (abs(ncp1).le.IQMX .and. abs(ncp2).le.IQMX ) 
     *       .or.( ncp1.eq.IGLU .and. ncp2.eq.IGLU ) ) then
               nv = nv + 1
               if( ncp1.ne.IGLU ) then
                 mvx(nv) = 0
               else
                 mvx(nv) = 3
               end if
               nadr1 = mksix1( nc1,np,ma )
               nadr2 = mksix1( nc2,np,ma )
               if ( ncp1.lt.0 ) then
                 call mksnvl(nvl,nv, nadr1, nadr2, 0 )
               else
                 call mksnvl(nvl,nv, nadr2, nadr1, 0 )
               end if
             end if
           end if

c        quark -->
           if( n1.gt.IGLU.and. n1.le.IQMX ) then
             nv = nv + 1
             nadr = mksix1( n,np,ma )
             if( ncp1.eq.IGLU .or. ncp2.eq.IGLU ) then
               mvx(nv) = 1
               nadr1 = mksix1( nc1,np,ma )
               nadr2 = mksix1( nc2,np,ma )
               if ( ncp1.eq.IGLU ) then
                 call mksnvl(nvl,nv, nadr, nadr2, nadr1 )
               else
                 call mksnvl(nvl,nv, nadr, nadr1, nadr2 )
               end if
             else 
               mvx(nv) = 0
               if ( ncp1.gt.IGLU .and. ncp1.le.IQMX ) then
                 nadr1 = mksix1( nc1,np,ma )
                 call mksnvl(nvl,nv, nadr, nadr1, 0 )
               else
                 nadr2 = mksix1( nc2,np,ma )
                 call mksnvl(nvl,nv, nadr, nadr2, 0 )
               end if
             end if
           end if

c        anti-quark -->
           if( n1.lt.IGLU.and. n1.ge.-IQMX ) then
             nv = nv + 1
             nadr = mksix1( n,np,ma )
             if( ncp1.eq.IGLU .or. ncp2.eq.IGLU ) then
               mvx(nv) = 1
               nadr1 = mksix1( nc1,np,ma )
               nadr2 = mksix1( nc2,np,ma )
               if ( ncp1.eq.IGLU ) then
                 call mksnvl(nvl,nv, nadr2, nadr, nadr1 )
               else
                 call mksnvl(nvl,nv, nadr1, nadr, nadr2 )
               end if
             else 
               mvx(nv) = 0
               if ( ncp1.lt.IGLU .and. ncp1.ge.-IQMX ) then
                 nadr1 = mksix1( nc1,np,ma )
                 call mksnvl(nvl,nv, nadr1, nadr, 0 )
               else
                 nadr2 = mksix1( nc2,np,ma )
                 call mksnvl(nvl,nv, nadr2, nadr, 0 )
               end if
             end if
           end if

c        gluon -->
           if( n1.eq.IGLU ) then
             nv = nv + 1
             nadr = mksix1( n,np,ma )
             nadr1 = mksix1( nc1,np,ma )
             nadr2 = mksix1( nc2,np,ma )
             if( ncp1.eq.IGLU .and. ncp2.eq.IGLU ) then
               mvx(nv) = 2
               call mksnvl(nvl,nv, nadr, nadr1, nadr2 )
             else 
               mvx(nv) = 1
               if ( ncp1.gt.IGLU .and. ncp1.le.IQMX ) then
                 call mksnvl(nvl,nv, nadr2, nadr1, nadr )
               else
                 call mksnvl(nvl,nv, nadr1, nadr2, nadr )
               end if
             end if
           end if

        end if 

c     3-body branch
        if ( n3.eq.3 ) then

c        photon --> gluon + remnant (q+qbar)
           if( n1.eq.IPH ) then
             nc1 = 0
             nc2 = 0
             nc3 = 0
             do 3 k=0,2
               n4k = n+n4+k
               if( listp(1,n4k).lt.IGLU .and.
     1             listp(1,n4k).ge.-IQMX) nc1=n4k
               if( listp(1,n4k).gt.IGLU .and.
     1             listp(1,n4k).le.IQMX ) nc2=n4k
               if( listp(1,n4k).eq.IGLU ) nc3=n4k
    3        continue
             if( nc1*nc2*nc3.ne.0 ) then
               nv = nv + 1
               mvx(nv) = 1
               nadr1 = mksix1( nc1,np,ma )
               nadr2 = mksix1( nc2,np,ma )
               nadr3 = mksix1( nc3,np,ma )
               call mksnvl(nvl,nv, nadr1, nadr2, nadr3 )
             end if
           end if
        end if 

  110 continue
c ---------------------------------------

c --- hard scattering 
c         nhrd1 -->--***-->-- nhrd3
c                    ***
c         nhrd2 -->--***-->-- nhrd4
c
c    nhdr* = real address in /qcevnt/
c    nhp*  = particle code 
c    nadr* = address in ma, mp, ms

c code of hard scatters
      nhp1 = listp(1,nhrd1)
      nhp2 = listp(1,nhrd2)

c partons after hard scattering
      nhrd3 = nhrd1 + listp(4,nhrd1)
      nhrd4 = nhrd2 + listp(4,nhrd2)
      nhp3 = listp(1,nhrd3)
      nhp4 = listp(1,nhrd4)
      if ( abs(nhp3).le.IQMX ) nadr3 = mksix1( nhrd3,np,ma )
      if ( abs(nhp4).le.IQMX ) nadr4 = mksix1( nhrd4,np,ma )

      if( iprt.gt.1 ) then
        write(6,602) nhrd1,nhrd2,nhrd3,nhrd4
        write(6,603) nhp1 ,nhp2 ,nhp3 ,nhp4
      end if


c  ( e-/e+ + q/g ; q/g + e-/e+ )
      if ( ( abs(nhp1).eq.IELE .and. abs(nhp2).le.IQMX ) .or.
     *     ( abs(nhp2).eq.IELE .and. abs(nhp1).le.IQMX ) ) then
        nv = nv + 1
        if( abs(nhp1).eq.IELE ) then
          kadr1 = mksix1( nhrd2,np,ma )
          kadr3 = nadr4
          nehp  = nhp2
        else
          kadr1 = mksix1( nhrd1,np,ma )
          kadr3 = nadr3
          nehp  = nhp1
        end if 

        if ( nehp.gt.IGLU ) then
          mvx(nv) = 0
          call mksnvl(nvl,nv, kadr1, kadr3, 0 )
        else if ( nehp.lt.IGLU ) then
          mvx(nv) = 0
          call mksnvl(nvl,nv, kadr3, kadr1, 0 )
        else
          mvx(nv) = 3
          call mksnvl(nvl,nv, kadr1, kadr3, 0 )
        end if


c   ( photon + photon )
      else if( nhp1.eq.IPH .and. nhp2.eq.IPH ) then
        nv = nv + 1
        mvx(nv) = 0
        if ( nhp3.lt.IGLU ) then
          call mksnvl(nvl,nv, nadr3, nadr4, 0 )
        else
          call mksnvl(nvl,nv, nadr4, nadr3, 0 )
        end if

c   ( photon + ... )
      else if( nhp1.eq.IPH ) then         
        nadr2 = mksix1( nhrd2,np,ma )

c   ( photon + gluon )
        if( nhp2.eq.IGLU ) then
          nv = nv + 1
          mvx(nv) = 1
          if ( nhp3.lt.IGLU ) then
            call mksnvl(nvl,nv, nadr3, nadr4, nadr2 )
          else
            call mksnvl(nvl,nv, nadr4, nadr3, nadr2 )
          end if

c   ( photon + quark )
        else if( nhp2.gt.IGLU ) then
          nv = nv + 1
          mvx(nv) = 1
          if ( nhp3.eq.IGLU ) then
            call mksnvl(nvl,nv, nadr2, nadr4, nadr3 )
          else
            call mksnvl(nvl,nv, nadr2, nadr3, nadr4 )
          end if

c   ( photon + antiquark )
        else if( nhp2.lt.IGLU ) then
          nv = nv + 1
          mvx(nv) = 1
          if ( nhp3.eq.IGLU ) then
            call mksnvl(nvl,nv, nadr4, nadr2, nadr3 )
          else
            call mksnvl(nvl,nv, nadr3, nadr2, nadr4 )
          end if

        end if

c   ( ... + photon )
      else if( nhp2.eq.IPH ) then         
        nadr1 = mksix1( nhrd1,np,ma )

c   ( gluon + photon )
        if( nhp1.eq.IGLU ) then
          nv = nv + 1
          mvx(nv) = 1
          if ( nhp3.lt.IGLU ) then
            call mksnvl(nvl,nv, nadr3, nadr4, nadr1 )
          else
            call mksnvl(nvl,nv, nadr4, nadr3, nadr1 )
          end if

c   ( quark + photon  )
        else if( nhp1.gt.IGLU ) then
          nv = nv + 1
          mvx(nv) = 1
          if ( nhp3.eq.IGLU ) then
            call mksnvl(nvl,nv, nadr1, nadr4, nadr3 )
          else
            call mksnvl(nvl,nv, nadr1, nadr3, nadr4 )
          end if

c   ( antiquark + photon  )
        else if( nhp1.lt.IGLU ) then
          nv = nv + 1
          mvx(nv) = 1
          if ( nhp3.eq.IGLU ) then
            call mksnvl(nvl,nv, nadr4, nadr1, nadr3 )
          else
            call mksnvl(nvl,nv, nadr3, nadr1, nadr4 )
          end if

        end if

c  ( qcd-type scatterng )
      else if ( abs(nhp1).le.IQMX .and. abs(nhp2).le.IQMX ) then
        nadr1 = mksix1( nhrd1,np,ma )
        nadr2 = mksix1( nhrd2,np,ma )

c  add a particle and two vertices
        np = np + 1
        ma( np ) = 0
        ms( np ) = 1
        nv1 = nv+1
        nv2 = nv+2
        nv = nv + 2

c       invariant mass
        ss = amksqq(nhrd1,nhrd2,1) 
        tt = amksqq(nhrd1,nhrd3,-1) 
        uu = amksqq(nhrd1,nhrd4,-1) 

c    ( gluon + gluon )
        if( nhp1.eq.IGLU .and. nhp2.eq.IGLU ) then

c      ( gluon + gluon -> gluon + gluon )
          if( nhp3.eq.IGLU ) then

            mp( np ) = 0
            mvx( nv1 ) = 2
            mvx( nv2 ) = 2
  
            if( abs(ss).le.abs(tt) .and. abs(ss).le.abs(uu) ) then
              call mksnvl(nvl,nv1, nadr1, nadr2, np  )
              call mksnvl(nvl,nv2, nadr4, nadr3, np  )
            end if
            if( abs(tt).le.abs(ss) .and. abs(tt).le.abs(uu) ) then
              call mksnvl(nvl,nv1, nadr3, nadr1, np  )
              call mksnvl(nvl,nv2, nadr2, nadr4, np  )
            end if
            if( abs(uu).le.abs(tt) .and. abs(uu).le.abs(ss) ) then
              call mksnvl(nvl,nv1, nadr4, nadr1, np  )
              call mksnvl(nvl,nv2, nadr2, nadr3, np  )
            end if

c      ( gluon + gluon -> q + qbar )
          else
            if( nhp3.gt.IGLU ) then
                kadr3 = nadr3
                kadr4 = nadr4
                ttt = tt
                uuu = uu
            else
                kadr3 = nadr4
                kadr4 = nadr3
                ttt = uu
                uuu = tt
            end if

            if( abs(ss).le.abs(tt) .and. abs(ss).le.abs(uu) ) then
              mp( np ) = 0
              mvx( nv1 ) = 2
              mvx( nv2 ) = 1
              call mksnvl(nvl,nv1, nadr1, nadr2, np  )
              call mksnvl(nvl,nv2, kadr4, kadr3, np  )
            end if
            if( abs(ttt).le.abs(ss) .and. abs(ttt).le.abs(uuu) ) then
              mp( np ) = abs(nhp3)
              mvx( nv1 ) = 1
              mvx( nv2 ) = 1
              call mksnvl(nvl,nv1, kadr4, np , nadr2  )
              call mksnvl(nvl,nv2, np , kadr3, nadr1  )
            end if
            if( abs(uuu).le.abs(ttt) .and. abs(uuu).le.abs(ss) ) then
              mp( np ) = abs(nhp3)
              mvx( nv1 ) = 1
              mvx( nv2 ) = 1
              call mksnvl(nvl,nv1, kadr4, np , nadr1  )
              call mksnvl(nvl,nv2, np , kadr3, nadr2  )
            end if
          end if

c    ( quark/anti-q + gluon )
        else if( nhp1.ne.IGLU .and. nhp2.eq.IGLU ) then

          if( abs(ss).le.abs(tt) .and. abs(ss).le.abs(uu) ) then
            mp( np ) = nhp1 
            mvx( nv1 ) = 1
            mvx( nv2 ) = 1
            if( nhp1.gt.IGLU ) then
              call mksnvl(nvl,nv1, nadr1, np , nadr2  )
              call mksnvl(nvl,nv2, np , nadr3, nadr4  )
            else
              call mksnvl(nvl,nv1, nadr3, np , nadr4  )
              call mksnvl(nvl,nv2, np , nadr1, nadr2  )
            end if
          end if
          if( abs(tt).le.abs(ss) .and. abs(tt).le.abs(uu) ) then
            mp( np ) = 0
            mvx( nv1 ) = 1
            mvx( nv2 ) = 2
            if( nhp1.gt.IGLU ) then
              call mksnvl(nvl,nv1, nadr1, nadr3, np  )
              call mksnvl(nvl,nv2, nadr2, nadr4, np  )
            else
              call mksnvl(nvl,nv1, nadr3, nadr1, np  )
              call mksnvl(nvl,nv2, nadr2, nadr4, np  )
            end if
          end if
          if( abs(uu).le.abs(tt) .and. abs(uu).le.abs(ss) ) then
            mp( np ) = nhp1 
            mvx( nv1 ) = 1
            mvx( nv2 ) = 1
            if( nhp1.gt.IGLU ) then
              call mksnvl(nvl,nv1, nadr1, np , nadr4  )
              call mksnvl(nvl,nv2, np , nadr3, nadr2  )
            else
              call mksnvl(nvl,nv1, nadr3, np , nadr2  )
              call mksnvl(nvl,nv2, np , nadr1, nadr4  )
            end if
          end if

c    ( gluon + quark/anti-q )
        else if( nhp1.eq.IGLU .and. nhp2.ne.IGLU ) then

          if( abs(ss).le.abs(tt) .and. abs(ss).le.abs(uu) ) then
            mp( np ) = nhp2 
            mvx( nv1 ) = 1
            mvx( nv2 ) = 1
            if( nhp2.gt.IGLU ) then
              call mksnvl(nvl,nv1, nadr2, np , nadr1  )
              call mksnvl(nvl,nv2, np , nadr4, nadr3  )
            else
              call mksnvl(nvl,nv1, nadr4, np , nadr3  )
              call mksnvl(nvl,nv2, np , nadr2, nadr1  )
            end if
          end if
          if( abs(tt).le.abs(ss) .and. abs(tt).le.abs(uu) ) then
            mp( np ) = 0
            mvx( nv1 ) = 1
            mvx( nv2 ) = 2
            if( nhp2.gt.IGLU ) then
              call mksnvl(nvl,nv1, nadr2, nadr4, np  )
              call mksnvl(nvl,nv2, nadr1, nadr3, np  )
            else
              call mksnvl(nvl,nv1, nadr4, nadr2, np  )
              call mksnvl(nvl,nv2, nadr1, nadr3, np  )
            end if
          end if
          if( abs(uu).le.abs(tt) .and. abs(uu).le.abs(ss) ) then
            mp( np ) = nhp2 
            mvx( nv1 ) = 1
            mvx( nv2 ) = 1
            if( nhp2.gt.IGLU ) then
              call mksnvl(nvl,nv1, nadr2, np , nadr3  )
              call mksnvl(nvl,nv2, np , nadr4, nadr1  )
            else
              call mksnvl(nvl,nv1, nadr4, np , nadr1  )
              call mksnvl(nvl,nv2, np , nadr2, nadr3  )
            end if
          end if

c    ( quark, antiquark )
        else
c      ( quark, antiquark , unequal flavor )
          if( abs(nhp1).ne.abs(nhp2) ) then

            mp( np ) = 0
            mvx( nv1 ) = 1
            mvx( nv2 ) = 1

            if ( nhp1.lt.IGLU ) then
              call mksnvl(nvl,nv1, nadr3, nadr1, np  )
            else
              call mksnvl(nvl,nv1, nadr1, nadr3, np  )
            end if

            if ( nhp2.lt.IGLU ) then
              call mksnvl(nvl,nv2, nadr4, nadr2, np  )
            else
              call mksnvl(nvl,nv2, nadr2, nadr4, np  )
            end if

c      ( q+q or qbar+qbar, equal flavor )
          else if( nhp1.eq.nhp2 ) then
            mp( np ) = 0
            mvx( nv1 ) = 1
            mvx( nv2 ) = 1

          if( abs(tt).le.abs(uu) ) then
            if( nhp1.gt.IGLU ) then
              call mksnvl(nvl,nv1, nadr1, nadr3, np  )
              call mksnvl(nvl,nv2, nadr2, nadr4, np  )
            else
              call mksnvl(nvl,nv1, nadr3, nadr1, np  )
              call mksnvl(nvl,nv2, nadr4, nadr2, np  )
            end if
          end if
          if( abs(uu).le.abs(tt) ) then
            if( nhp1.gt.IGLU ) then
              call mksnvl(nvl,nv1, nadr1, nadr4, np  )
              call mksnvl(nvl,nv2, nadr2, nadr3, np  )
            else
              call mksnvl(nvl,nv1, nadr4, nadr1, np  )
              call mksnvl(nvl,nv2, nadr3, nadr2, np  )
            end if
          end if

c      ( q+qbar , equal flavor )
          else 
            if( nhp1.gt.IGLU ) then
                kadr1 = nadr1
                kadr2 = nadr2
                ttt = tt
                uuu = uu
            else
                kadr1 = nadr2
                kadr2 = nadr1
                ttt = uu
                uuu = tt
            end if

c             ( q+qbar-> gluon+gluon , equal flavor )
            if ( nhp3.eq.IGLU .and. nhp4.eq.IGLU ) then
              if( abs(ss).le.abs(tt) .and. abs(ss).le.abs(uu) ) then
                mp( np ) = 0
                mvx( nv1 ) = 1
                mvx( nv2 ) = 2
                call mksnvl(nvl,nv1, kadr1, kadr2, np  )
                call mksnvl(nvl,nv2, nadr4, nadr3, np  )
              end if
              if( abs(ttt).le.abs(ss) .and. abs(ttt).le.abs(uuu) ) then
                mp( np ) = abs(nhp1)
                mvx( nv1 ) = 1
                mvx( nv2 ) = 1
                call mksnvl(nvl,nv1, np , kadr2, nadr4  )
                call mksnvl(nvl,nv2, kadr1, np , nadr3  )
              end if
              if( abs(uuu).le.abs(ttt) .and. abs(uuu).le.abs(ss) ) then
                mp( np ) = abs(nhp1)
                mvx( nv1 ) = 1
                mvx( nv2 ) = 1
                call mksnvl(nvl,nv1, np , kadr2, nadr3  )
                call mksnvl(nvl,nv2, kadr1, np , nadr4  )
              end if

c             ( q+qbar->q+qbar , equal flavor )
            else
              if( nhp3.gt.IGLU ) then
                  kadr3 = nadr3
                  kadr4 = nadr4
              else
                  kadr3 = nadr4
                  kadr4 = nadr3
              end if
              if( nhp1*nhp3 .gt. IGLU ) then
                ut = tt
              else
                ut = uu
              end if

              mp( np ) = 0
              mvx( nv1 ) = 1
              mvx( nv2 ) = 1
    
              if( abs(ss).le.abs(ut) ) then
                  call mksnvl(nvl,nv1, kadr1, kadr2, np  )
                  call mksnvl(nvl,nv2, kadr4, kadr3, np  )
              end if
              if( abs(ut).le.abs(ss) ) then
                  call mksnvl(nvl,nv1, kadr1, kadr3, np  )
                  call mksnvl(nvl,nv2, kadr4, kadr2, np  )
              end if
            end if

          end if

        end if

c  ( error )
      else
        write(6,9005) nhp1,nhp2
        go to 9990
      end if

      if( iprt.gt.1 ) then
        write(6,604)
        call mksdm1(np,nv,ma,mp,ms,mvx,nvl)
      end if

c ---------------------------------------
 
c --- reduction (step-1)
c     3-gluon vertex --> quark loop
c      ( and g-g-photon is also handled here )
c     probability = 1:1 

c       1(g)                    1
c        I                      I nv0
c        I          =>       ---*---
c        I                np1I     Inp
c 2(g)   I   3(g/ph)     2   I np2 I   3
c   -----*-----          ----*-----*----
c       nv0                 nv1   nv2

c temporary size of vertex table
      nvtmp = nv

      do 210 n=1, nv

      if ( mvx(n).eq.2 .or. mvx(n).eq.3 ) then

        np1 = np+1 
        np2 = np+2 
        np  = np+3
        ma( np1 ) = 0
        ma( np2 ) = 0
        ma( np  ) = 0
        mp( np1 ) = 1
        mp( np2 ) = 1
        mp( np  ) = 1
        ms( np1 ) = 1
        ms( np2 ) = 1
        ms( np  ) = 1

        nv0 = n
        nv1 = nvtmp+1
        nv2 = nvtmp+2
        nvtmp = nv2
c                              g-g-ph vertex case
        if( mvx(n).eq.3 ) then
          mvx( nv2 ) = 0
          mggph = mggph+1
          nggph( mggph ) = nv0
          mggph = mggph+1
          nggph( mggph ) = nv1
        else
        mvx( nv2 ) = 1
        end if
        mvx( nv0 ) = 1
        mvx( nv1 ) = 1

        nvl1 = nvl(1,n)
        nvl2 = nvl(2,n)
        nvl3 = nvl(3,n)

        r = RANDOM( 0 )
        if ( r.lt.0.5d0 ) then
          call mksnvl(nvl,nv0, np  , np1 , nvl1 )
          call mksnvl(nvl,nv1, np1 , np2 , nvl2 )
          call mksnvl(nvl,nv2, np2 , np  , nvl3 )
        else
          call mksnvl(nvl,nv0, np1 , np  , nvl1 )
          call mksnvl(nvl,nv1, np2 , np1 , nvl2 )
          call mksnvl(nvl,nv2, np  , np2 , nvl3 )
        end if

      end if

  210 continue

      nv = nvtmp

c --- reduction (step-2)
c     internal gluon -- > 2 quark lines
c     probability = 1:9 ... parallel : crossed
c
c     --(1)->--(nvg1)-->-(2)--
c                I
c               (3) ... gluon at n
c                I
c     --(1)->--(nvg2)-->-(2)--

      do 250 n=1, np 

      if ( mp(n).eq.0 .and. ms(n).eq.1 ) then

        nvg1 = 0
        nvg2 = 0

        do 40 k=1,nv
        if ( mvx(k) .eq. 1 ) then
          if ( nvl(3,k).eq.n ) then
            if ( nvg1.eq.0 ) then
              nvg1 = k
            else
              nvg2 = k
            end if
          end if
        end if
   40   continue

        if( nvg1.eq.0 .or. nvg2.eq.0 ) then
          write(6,9006) n
          go to 9990
        end if
   
        mvx( nvg1 ) = 0
        mvx( nvg2 ) = 0
        nvl(3,nvg1) = 0
        nvl(3,nvg2) = 0

        ms( n ) = -2

        r = RANDOM( 0 )
c                   g-g-photon related--> force crossed
c                   if parallel, one-gluon loop might appear
        if ( mggph.gt.0 ) then
          do 15 j=1,mggph
            if ( nvg1.eq.nggph(j) .or. nvg2.eq.nggph(j) ) r=1.0d0
   15     continue
        end if
        if ( mretry.eq.NRETRY ) r=1.0d0

        if ( r.lt.0.1111111d0 ) then
        else
          kk = nvl(2,nvg1)
          nvl(2,nvg1)= nvl(2,nvg2)
          nvl(2,nvg2)= kk
        end if

      end if

  250 continue

      if( iprt.gt.1 .and. KDBG.eq.1 ) then
        write(6,605)
        call mksdm1(np,nv,ma,mp,ms,mvx,nvl)
      end if
c ---------------------------------------

c --- construct color singlet

      nstr = 0
      nadr = 0

      do 50 n=1,nv
   50 mvxx(n)=1

c quark strings

      do 300 n=1,nv

c                                   used ?
        if( mvxx(n).eq.0 ) go to 300

c                    start point quark
        k1 = nvl(1,n)
        do 60 n1=1,nv
          if( nvl(2,n1).eq.k1 ) go to 300
   60   continue

c                 found starting point
        npntr = n

        nstr = nstr+1
        kstr(nstr) = 1
c                record starting quark
        nadr = nadr+1
        nastr(nadr) = k1
        mstr(nstr) = nadr
        
   62   continue

c                record gluon for q-q-g vertex
        if ( mvx(npntr).eq.1 ) then
          nadr = nadr+1
          nastr(nadr) = nvl(3,npntr)
        end if

c                 outgoing quark
        k1 = nvl(2,npntr) 
        mvxx(npntr) = 0

c                              end point ?
        if( ms(k1).eq.0 ) then
              nadr = nadr+1
              nastr(nadr) = k1
              lstr(nstr)=nadr-mstr(nstr)+1
              go to 300
        end if

c                     next vertex
        do 64 n1=1,nv
          if( mvxx(n1).eq.0 ) go to 64
          if( nvl(1,n1).eq.k1 ) then 
            npntr = n1
            go to 62 
          end if
   64   continue
 
  300 continue

c gluonic loops

      do 310 n=1,nv

c                                   used ?
        if( mvxx(n).eq.0 ) go to 310

c                    start point of loop 
        k1 = nvl(1,n)
        ks = k1
        npntr = n
        nstr = nstr+1
        kstr(nstr) = 0
        mstr(nstr) = nadr+1
        
   72   continue

c                record gluon for q-q-g vertex
        if ( mvx(npntr).eq.1 ) then
          nadr = nadr+1
          nastr(nadr) = nvl(3,npntr)
        end if

c                 outgoing quark
        k1 = nvl(2,npntr) 
        mvxx(npntr) = 0

c                              end point ?
        if( k1.eq.ks ) then
              lstr(nstr)=nadr-mstr(nstr)+1
              go to 310
        end if

c                     next vertex
        do 74 n1=1,nv
          if( mvxx(n1).eq.0 ) go to 74
          if( nvl(1,n1).eq.k1 ) then 
            npntr = n1
            go to 72 
          end if
   74   continue
 
  310 continue

c  check if any short gluonic loop
        do 19 j=1,nstr
        if( lstr(j).lt.2 ) then
            mretry = mretry + 1
            if(KDBG.eq.1) write(6,*) 'RETRY',mretry,' ',NRETRY
            if( mretry.le.NRETRY ) go to 99
        end if
   19   continue


c ---  external address  
       do 330 n=1,nadr
         j = nastr(n)
         if( j.le.0.or.j.gt.np ) then
           write(6,9009) n
           go to 9990
         end if
         if( ma(j).le.0.or.ma(j).gt.ntop ) then
           write(6,9010) n
           go to 9990
         end if
         nastr(n) = ma(j)
  330  continue

c ---------------------------------------

c --- report final results
      if ( iprt.ge.1 ) then
        write(6,610) nstr
        do 90 n=1,nstr
        write(6,611) n,kstr(n),lstr(n),mstr(n)
          j1 = mstr(n)
          j2 = mstr(n)+lstr(n)-1
        write(6,612) listp(1,nastr(j1)),listp(1,nastr(j2)),
     1     (nastr(j),j=j1,j2)
   90   continue
      end if

c --- final check
      if( nstr.le.0 ) then
         write(6,9007)
         go to 9990
      end if

      do 95 n=1,nstr
        if( lstr(n).lt.2 ) then
          write(6,9008) n
          go to 9990
        end if
   95 continue
c ---------------------------------------
      nret = 0
      return

c ---------------------------------------
c             Error return
 9990 continue
      write(6,9980) 
      write(6,601) np,nqu,naq,ngl
      call mksdm1(np,nv,ma,mp,ms,mvx,nvl)
      write(6,602) nhrd1,nhrd2,nhrd3,nhrd4
      write(6,603) nhp1 ,nhp2 ,nhp3 ,nhp4
      write(6,610) nstr
      do 91 n=1,nstr
      write(6,611) n,kstr(n),lstr(n),mstr(n)
      write(6,612) (nastr(j),j=mstr(n),mstr(n)+lstr(n)-1)
   91 continue
      call qclist
      nret = 1
      return
c ---------------------------------------
  601 format('(mkstr3) np=',i3,'  quark, anti-q, gluon =',3i5)
  602 format('(mkstr3) hard scattering : ',2i5,' -> ',2i5)
  603 format('                           ',2i5,' -> ',2i5)
  604 format('(mkstr3) before reduction') 
  605 format('(mkstr3) after  reduction') 
  610 format('(mkstr3) number of color singlets,  nstr=',i5)
  611 format('  ',i3,'-th string, type(1/0..q/g)=',i2,
     * ' length=',i3,' start=',i3)
  612 format(' (',i2,':',i2,')  order= ',15i4)
c ---------------------------------------
 9001 format('(mkstr3) q <> qbar',2i5)
 9002 format('(mkstr3) third hard-scatter ?  at n=',i3)
 9003 format('(mkstr3) hard scatters not found')
 9005 format('(mkstr3) strange combination for hard scatt. ',2i10)
 9006 format('(mkstr3) gluon bridge failed, n=',i10) 
 9007 format('(mkstr3) No string can be created.')
 9008 format('(mkstr3) Short string at n=',i5)
 9009 format('(mkstr3) Bad code (9009) at n=',i5)
 9010 format('(mkstr3) Bad code (9010) at n=',i5)
 9980 format('(mkstr3) Error. Please report above line and',
     * ' following list.  Return with nret = 1.')
      end


c ----------------------------------------------------------
c  mksix1 : Table look-up function -(1)
c ----------------------------------------------------------
      integer function mksix1(nad,np,ma)

c     mkstr2 --> mksix1
c   For an address in /qcevnt/,  return position in 
c   wroking tables is mkstr2.
c
c arguments:
c   nad : IN : address in /qcevnt/
c   np  : IN : size of table ma
c   ma(i) : IN : reference table in mkstr2.
c
c returns:
c       ma( mksix1 ) = nad
c

      dimension ma(1)

      do 10 k = 1,np
        if ( ma(k) .eq.nad )  then
          mksix1 = k
          return
        end if
   10 continue

      write(6,9001) nad,np
      mksix1 = 0

      return
 9001 format('(mksix1) Cannot find. nad,np=',2i10)
      end

c ----------------------------------------------------------
c  mksnvl : Set values into nvl
c ----------------------------------------------------------
      subroutine mksnvl(nvl,nv,nvl1,nvl2,nvl3)

      dimension nvl(3,1)

      nvl(1,nv) = nvl1
      nvl(2,nv) = nvl2
      nvl(3,nv) = nvl3

      return
      end


c ----------------------------------------------------------
c  amksqq :  Compute (p_1+p_2)**2
c ----------------------------------------------------------
      double precision function amksqq(nad1,nad2,is)

      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/qcevnt/partn(10,1000),listp(10,1000),NTOP

      amksqq = -(partn(4,nad1)+is*partn(4,nad2))**2
     1         -(partn(5,nad1)+is*partn(5,nad2))**2
     2         -(partn(6,nad1)+is*partn(6,nad2))**2
     3         +(partn(7,nad1)+is*partn(7,nad2))**2

      return
      end


c ----------------------------------------------------------
c  mksdm1 : Dump routine for development
c ----------------------------------------------------------
      subroutine mksdm1(np,nv,ma,mp,ms,mvx,nvl)

      parameter( NC=12 )
      dimension ma(1),mp(1),ms(1),mvx(1),nvl(3,1)

      write(6,*) '---------- dump from mkstr3; np,nv=',np,',',nv
      if( np.gt.0 ) then
        nn= (np-1)/NC
        do 10 j=0,nn
          n1=j*NC+1
          n2=(j+1)*NC
          if ( n2.gt.np ) n2=np
          write(6,101) (n,n=n1,n2)
          write(6,102) (ma(n),n=n1,n2)
          write(6,103) (mp(n),n=n1,n2)
          write(6,104) (ms(n),n=n1,n2)
   10   continue
      end if
      write(6,*) ' '

      if( nv.gt.0 ) then
        nn= (nv-1)/NC
        do 20 j=0,nn
          n1=j*NC+1
          n2=(j+1)*NC
          if ( n2.gt.nv ) n2=nv
          write(6,110) (n,n=n1,n2)
          write(6,105) (mvx(n),n=n1,n2)
          do 15 k=1,3
   15     write(6,106) k,(nvl(k,n),n=n1,n2)
   20   continue
      end if
      write(6,*) ' '

      return
  101 format('   /',12i5)
  102 format(' ma:',12i5)
  103 format(' mp:',12i5)
  104 format(' ms:',12i5)
  110 format('       /',12i5)
  105 format(' mvx   :',12i5)
  106 format(' nvl',i2,' :',12i5)
      end
